@@ -157,6 +157,13 @@ class TestPyramidBeakerSessionObject(unittest.TestCase):
         self.assertEqual(token, 'token')
         self.failUnless('_csrft_' in session)
 
+    def test_get_csrf_token_new(self):
+        request = DummyRequest()
+        session = self._makeOne(request)
+        token = session.get_csrf_token()
+        self.failUnless(token)
+        self.assertEqual(session['_csrft_'], token)
+
 class Test_session_factory_from_settings(unittest.TestCase):
     def _callFUT(self, settings):
         from pyramid_beaker import session_factory_from_settings
@@ -166,7 +173,17 @@ class Test_session_factory_from_settings(unittest.TestCase):
         settings = {'session.auto':'true', 'session.key':'foo'}
         factory = self._callFUT(settings)
         self.assertEqual(factory._options, {'auto':True, 'key':'foo'})
-    
+
+    def test_cookie_on_exception_true(self):
+        settings = {'session.cookie_on_exception':'true'}
+        factory = self._callFUT(settings)
+        self.assertEqual(factory._cookie_on_exception, True)
+
+    def test_cookie_on_exception_false(self):
+        settings = {'session.cookie_on_exception':'false'}
+        factory = self._callFUT(settings)
+        self.assertEqual(factory._cookie_on_exception, False)
+
 class DummyRequest:
     def __init__(self):
         self.callbacks = []
@@ -178,7 +195,43 @@ class DummyRequest:
 class DummyResponse:
     def __init__(self):
         self.headerlist = []
-        
+    
+class Test_session_cookie_on_exception(unittest.TestCase):
+
+    def _makeOne(self, request, **options):
+        from pyramid_beaker import BeakerSessionFactoryConfig
+        return BeakerSessionFactoryConfig(**options)(request)
+
+    def test_default_cookie_on_exception_setting(self):
+        request = DummyRequest()
+        session = self._makeOne(request)
+        self.assertEqual(session._cookie_on_exception, False)
+
+    def test_cookie_on_exception_setting(self):
+        request = DummyRequest()
+        session = self._makeOne(request, cookie_on_exception=True)
+        self.assertEqual(session._cookie_on_exception, True)
+    
+    def _assert_session_persisted(self, request, session, expected):
+        # Checking response headers not likely best method of asserting 
+        # if Beaker's SessionObject.persist method was called.
+        # Not sure of best method of doing this.
+        request.exception = True 
+        session['use it'] = True
+        response = DummyResponse()
+        request.callbacks[0](request, response)
+        self.assertEqual(len(response.headerlist) == 1, expected)
+
+    def test_request_call_back_without_cookie_on_exception(self):
+        request = DummyRequest()
+        session = self._makeOne(request)
+        self._assert_session_persisted(request, session, False)
+
+    def test_request_call_back_with_cookie_on_exception(self):
+        request = DummyRequest()
+        session = self._makeOne(request, cookie_on_exception=True)
+        self._assert_session_persisted(request, session, True)
+
 class TestCacheConfiguration(unittest.TestCase):
     def _set_settings(self):
         return {'cache.regions':'default_term, second, short_term, long_term',
@@ -207,8 +260,8 @@ class TestCacheConfiguration(unittest.TestCase):
         del settings['cache.default_term.expire']
         set_cache_regions_from_settings(settings)
         default_term = beaker.cache.cache_regions.get('default_term')
-        self.assertEqual(default_term, {'expire': 60, 'type': 'memory',
-                                      'lock_dir': None})
+        self.assertEqual(default_term, {'url': None, 'expire': 60,
+                                      'type': 'memory', 'lock_dir': None})
     
     def test_add_cache_multiple_region(self):
         from pyramid_beaker import set_cache_regions_from_settings
@@ -234,3 +287,20 @@ class TestCacheConfiguration(unittest.TestCase):
                          settings['cache.lock_dir'])
         self.assertEqual(default_term.get('type'),
                          settings['cache.default_term.type'])
+    
+    def test_region_inherit_url(self):
+        from pyramid_beaker import set_cache_regions_from_settings
+        import beaker
+        settings = self._set_settings()
+        beaker.cache.cache_regions = {}
+        settings['cache.regions'] = 'default_term, short_term'
+        settings['cache.lock_dir'] = 'foo'
+        settings['cache.url'] = '127.0.0.1'
+        settings['cache.short_term.expire'] = '60'
+        settings['cache.default_term.type'] = 'file'
+        settings['cache.default_term.expire'] = '300'
+        set_cache_regions_from_settings(settings)
+        default_term = beaker.cache.cache_regions.get('default_term')
+        short_term = beaker.cache.cache_regions.get('short_term')
+        self.assertEqual(short_term.get('url'), settings['cache.url'])
+        self.assertEqual(default_term.get('url'), settings['cache.url'])
